% ===================== Precipitation GSM-Fusion Demo  ====================
% This set of Matlab codes can be used for multi_sensor multi-platform 
% Fusionof remote sensing precipitation data, using GSM probability model
% in the wavelet domain. The methodology is designed in a way that can 
% incorporatemutual error covariance structure of rain-gauge, NEXRAD and 
% TRMM data.Detail explanation of the methodology can be found in Ebtehaj 
% and Foufuola-Georgiou, [2011b].  
% Mohammad Ardeshir Ebtehaj, Jan, 2011, Rev00
% Email:ebteh001@umn.edu

%% ====================  Input Rainfall Images  ===========================
clear, close
load('.\Data\HSTN_2A25_19980628_181300.mat'); % loading TRMM data
load('.\Data\HSTN_N0Z_19980628_181300.mat');  % loading NEXRAD data
subplot 211
imagesc(im_nex,[0 50]); title('NEXRAD Ref. Image'); 
set(gca, 'XTick', [], 'YTick', []); 
xlabel(['Size:[',num2str(size(im_nex)),'];',' Range:[',num2str([min(im_nex(:)) ceil(max(im_nex(:)))]),']']);
colorbar('east')
subplot 212
imagesc(im_trm,[0 50]); title('TRMM Ref. Image'); 
set(gca, 'XTick', [], 'YTick', []); 
xlabel(['Dim:[',num2str(size(im_trm)),'];',' Range:[',num2str([min(im_trm(:)) ceil(max(im_trm(:)))]),']']);

%% =====================  Inputs Parameter  ===============================

% snr_rn=snr_rt=24.0; snr_nt=16.0 dB --> Figure 11 of the paper
% snr_rn=snr_rt=e.g.,60(large); snr_nt=11.0 dB --> Table 2 of the paper
sig_er=@(im,snr) std2(im(im>0))*10^(-snr/20);
snr_rn=60.0;   sig_vrn=sig_er(im_nex,snr_rn);   % rain gauges versus NEXRAD observation error 
snr_rt=60.0;   sig_vrt=sig_er(im_trm,snr_rt);   % rain gauges versus TRMM observation error
snr_nt=11.0;    sig_vnt=sig_er(im_nex,snr_nt);  % TRMM versus NEXRAD observation error
total_er=sig_vrn+sig_vrt+sig_vnt;
Nsc=4;                                          % Level of wavelet decompositions 
nhood=3;                                        % size of the neighborhood e.g. 3x3 for high-pass filtering
nhood_l=3;                                      % size of the neighborhood e.g. 5x5 for low-pass filtering
%% =====================  Fusion Process  =================================
clc
% Filtering rain-gauge error from NEXRAD image
disp('NEXRAD and Rain-Gauge Fusion... >>')
[im_dnois_nex]=WGSM_Precip_Filt(im_nex,Nsc,sig_vrn,nhood);  % snr_rn=10
% Filtering rain-gauge error from TRMM image
disp('TRMM and Rain-Gauge Fusion... >>')
[im_dnois_trm]=WGSM_Precip_Filt(im_trm,Nsc-2,sig_vrt,nhood); % snr_rn=10
% Projevction of the TRMM onto the linear subpspace of NEXRAD data
disp('TRMM and NEXRAD Fusion... >>')
[im_fused]=WGSM_TR_NEX_Proj(im_dnois_nex,im_dnois_trm,Nsc-2,sig_vnt,nhood_l,0); % sig_vnt=5.5;

%% =====================  Graphing the Results  ===========================
close all
subplot 311
imagesc(im_trm,[0 50]); title('TRMM');
set(gca, 'XTick', [], 'YTick', []); 
xlabel(['Size:[',num2str(size(im_trm)),'];',...
    ' Range:[',num2str([min(im_trm(:)) ceil(max(im_dnois_trm(:)))]),']']);
colorbar
subplot 312 
imagesc(im_dnois_nex,[0 50]); title('NEXRAD');
set(gca, 'XTick', [], 'YTick', []); 
xlabel(['Size:[',num2str(size(im_nex)),'];',...
    ' Range:[',num2str([min(im_nex(:)) ceil(max(im_nex(:)))]),']']);
subplot 313
imagesc(im_fused,[0 50]); title('GSM-Fused Product');
set(gca, 'XTick', [], 'YTick', []); 
xlabel(['Size:[',num2str(size(im_fused)),'];',...
    ' Range:[',num2str([min(im_fused(:)) ceil(max(im_fused(:)))]),']']);
% Table of data
% Computing Entropy Measure
im_trm_us=imresize(im_trm,4,'nearest'); % upscaled trmm
En_pmax=max([entropy(im_fused),entropy(im_nex),entropy(im_trm_us)]);
NEn_trm=entropy(im_trm_us)/En_pmax;
NEn_nex=entropy(im_nex)/En_pmax;
NEn_fus=entropy(im_fused)/En_pmax;
% Computing F-norms of Images
Fn_max=max([norm(im_trm_us,'fro'),norm(im_nex,'fro'),norm(im_fused,'fro')]);
NFn_trm=norm(im_trm_us,'fro')/Fn_max;
NFn_nex=norm(im_nex,'fro')/Fn_max;
NFn_fus=norm(im_fused,'fro')/Fn_max;
% Rain rate @ 6min 
% computation of the rain-rate based on Z=300R^(1.4)
R=@ (dbz) (10.^(dbz./10)./300).^(1/1.4).*(dbz>0);
im_fus_R=R(im_fused); im_trm_R=R(im_trm_us); im_nex_R=R(im_nex);
% rain-rate mean compuation 
im_trm_Rm=mean(im_trm_R(im_fused>0));
im_nex_Rm=mean(im_nex_R(im_fused>0));
im_fus_Rm=mean(im_fus_R(im_fused>0));
% rain-rate max computation
im_trm_Rmx=max(im_trm_R(im_fused>0));
im_nex_Rmx=max(im_nex_R(im_fused>0));
im_fus_Rmx=max(im_fus_R(im_fused>0));
% Generating Table of Information
f = figure('Position',[200 200 500 125]);
dat = [NEn_trm NEn_nex NEn_fus;NFn_trm NFn_nex NFn_fus;im_trm_Rm...
    im_nex_Rm im_fus_Rm;im_trm_Rmx im_nex_Rmx im_fus_Rmx];   
cnames = {'TRMM-Data','NEX-Data','Fus-Data'};
rnames = {'Normalized Entropy','Normalized F-Norm','Mean Rain rate [mm/hr]','Max Rain Rate [mm/hr]'};
t = uitable('Parent',f,'Data',dat,'ColumnName',cnames,... 
            'RowName',rnames,'Position',[20 20 470 95]);
% ============================  END of Demo  ==============================
